#
# Setup: Required environment variable, pysynphot, pylab
#
import os
os.environ['PYSYN_CDBS']=os.getcwd()+'/cdbs/'
import pysynphot as S
import pylab as P


# S.commands are pysynphot
# P.commands are matplotlib/pylab.
#
# The basic building blocks are Spectrum and Bandpass 
# We'll look at the Spectrum first.
# A spectrum is commonly read in from a file:

vega=S.FileSpectrum('data/alpha_lyr_stis_003.fits')


#All spectra have .wave and .flux that can be printed or plotted.
P.clf()
P.plot(vega.wave,vega.flux)
P.xlim(0,10000)

#They also know their own units and "name".
#Flux and wave units are defined by the file in this case.
#Plots can easily be saved to files; matplotlib supports ps, png, & others.
P.xlabel(vega.waveunits)
P.ylabel(vega.fluxunits)
P.title(vega.name)
P.savefig('vegaspec.ps')

#We can also create analytic spectra
#A black body
bb=S.BlackBody(40000)
print bb

#A power law
pow=S.PowerLaw(10000,-2)
print pow

#A gaussian
gss=S.GaussianSource(8.3e-12,18000,20,fluxunits='flam')
print gss

#A flat spectrum
onecount=S.FlatSpectrum(1,fluxunits='counts')
print onecount                   


#We can add and multiply spectra, even if they were
#created in different units
print bb.fluxunits
print gss.fluxunits
sp = bb + gss*4
print sp.fluxunits

#and look at the results
P.clf()
P.plot(sp.wave,sp.flux)
P.xlabel(sp.waveunits)
P.ylabel(sp.fluxunits)

#We can "convert" a spectrum to the units we want. The .wave and .flux
#attributes will be returned in the specified units.
sp.convert('flam')
P.clf()
P.plot(sp.wave,sp.flux)
P.xlabel(sp.waveunits)
P.ylabel(sp.fluxunits)



#Now let's look at some bandpasses.
#A bandpass can also be read in from a file.

ota=S.FileBandpass(os.path.expandvars('$PYSYN_CDBS/comp/ota/hst_ota_007_syn.fits'))
ccd=S.FileBandpass(os.path.expandvars('$PYSYN_CDBS/comp/acs/acs_hrc_ccd_013_syn.fits'))

#All bandpasses have .wave and .throughput that can be printed or plotted,
#and know their own name.
P.clf()
P.plot(ota.wave,ota.throughput)
P.xlabel(ota.waveunits)
P.title(ota.name)

#We can also create a bandpass from an obsmode string, as in SYNPHOT,
#and see the list of files used to create it
bp = S.ObsBandpass('acs,hrc,f555w')
bp.showfiles()

#Bandpasses can be multiplied
bp2=ota*ccd
print bp2

#and overplotted.
P.clf()
P.plot(bp.wave,bp.throughput)
P.plot(ota.wave,ota.throughput)
P.plot(ccd.wave,ccd.throughput)
P.plot(bp2.wave,bp2.throughput)
P.legend(['acs,hrc,f555w','ota','ccd','ota*ccd'])

#You can also create a box
mybox = S.Box(8000,1000)*0.8
P.plot(mybox.wave,mybox.throughput)

#You can query a bandpass at a given wavelength, which is
#the same functionality as calcphot.refwave and calcphot.tlambda
#The same functionality for spectrum objects is in progress.
print bp.sample(5000)
                   
#A spectrum can be renormalized in a given bandpass
rnbb= bb.renorm(17.0,'abmag',bp)
P.clf()
P.plot(rnbb.wave,rnbb.flux)
P.ylabel(rnbb.fluxunits)
P.xlabel(rnbb.waveunits)
P.title(rnbb)

#It can also be redshifted
zbb = rnbb.redshift(0.8)
P.plot(zbb.wave,zbb.flux)
P.legend(['bb','bb (z=0.8'])
P.xlim(0,20000)

#and extinction can be applied.
#By default, a diffuse Milky Way law is used.
mwext=S.Extinction(0.3)
print mwext.name

#but others are available
lmcext=S.Extinction(0.3,'lmcavg')
print lmcext.name

#Apply extinction to a spectrum by multiplying
sp2=vega*lmcext
P.clf()
P.plot(vega.wave,vega.flux)
P.plot(sp2.wave,sp2.flux)
P.xlim(0,10000)
P.xlabel(sp2.waveunits)
P.ylabel(sp2.fluxunits)
P.legend(['vega','vega*LMCAvg(0.3)'])


#So much for spectra and bandpasses alone; now let's talk
#about combining them.
#
#Once you have created the spectrum you want,
#(so let's make a new one and look at it)
sp=zbb+S.GaussianSource(1e-13,12000,300,fluxunits='flam')
P.clf()
P.plot(sp.wave,sp.flux)
P.xlabel(sp.waveunits)
P.ylabel(sp.fluxunits)
P.title(sp)

#and the bandpass you're interested in
nic=S.ObsBandpass('nicmos,3,g206')
P.clf()
P.plot(nic.wave,nic.throughput)
P.title(nic)

# you can simulate an observation of that spectrum through that bandpass
obs=S.Observation(sp,nic)
P.clf()
P.plot(obs.wave,obs.flux)
P.ylabel(obs.fluxunits)
P.xlabel(obs.waveunits)
print obs

#Observations can calculate their own countrate and
#effective wavelength
print obs.countrate()
print obs.efflam()

#When plotting in counts, use the binned wave and flux arrays
# - corresponds to the "optimal binning" from the countrate task
P.clf()
obs.convert('counts')
P.plot(obs.binwave,obs.binflux,'.-')
P.ylabel(obs.fluxunits)

#All these entities can be written to files
sp.writefits('zbb_em.fits')
nic.writefits(nic.name.replace(',','_'))
obs.writefits('obs.fits')


#Observations will be written using the binned flux by default,
#but you can specify the native binning
obs.writefits('obs_native.fits',binned=False)
#and see the files we've written
import glob
glob.glob("*.fits")

#With just a little bit of python, we can get IO savings in
#exploring parameter space:
#define bandpasses & spectra once upfront, then revisit them without
#paying the assembly cost again. EG, make a bunch of bandpasses
d={}
flist=['f555w', 'f606w', 'f625w']
for filt in flist:
    d[filt]=S.ObsBandpass('acs,hrc,%s'%filt)


#then apply them all to the same spectrum

o={}
for filt in flist:
    o[filt]=S.Observation(sp,d[filt])
P.clf()
for filt in flist:
    P.plot(o[filt].wave,o[filt].flux)
P.xlim(4000,8000)
P.legend(flist)

#Oh, let's add Johnson V for comparison
jv=S.ObsBandpass('johnson,v')
flist.append('johnson V')
obs=S.Observation(sp,jv)

#and overplot
P.plot(obs.wave,obs.flux)
P.xlim(4000,8000)
P.legend(flist)

#Synphot "expert" issues:
#Kurucz-style models are still supported, though we recommend you use
#the newer Castelli-Kurucz models. The lookup syntax is the same:
#Icat(dirname, Teff, metallicity, log_g)

g2star=S.Icat('ck04models',5860,0.0,4.4)
P.clf()
P.plot(g2star.wave,g2star.flux)
P.ylabel(g2star.fluxunits)
P.xlim(0,30000)
P.title(g2star)

#Parameterized obsmodes are supported
acs=S.ObsBandpass('acs,wfc1,f850lp')
acs_aper=S.ObsBandpass('acs,wfc1,aper#0.2,f850lp')
P.clf()
P.plot(acs.wave,acs.throughput)
P.plot(acs_aper.wave,acs_aper.throughput)
P.xlim(7000,12000)
P.legend(['f850lp','f850lp,aper#0.2'])

#Not yet implemented:
tbd="""Calcphot functionality
sp.effstim()
obs.effstim('abmag')
obs.rmslam()
...and so forth.

#and bandpar functionality
bp.fwhm()
bp.pivot()
...and so forth.

#Colors (eg, B-V) are also not yet implemented.
"""


